<?php
namespace Teknisa\CADFIS\Strategies;

use Teknisa\Libs\Util\Environment;
use Teknisa\CADFIS\Strategies\GeneralStrategy;
use Teknisa\CADFIS\Services\FormPgtoService as FormPgtoService;

class FormPgtoStrategy extends GeneralStrategy implements Interfaces\IStrategy
{
    protected $service;
    protected $errors = array();
    
    public function __construct(FormPgtoService $service, Environment $environment)
    {
		$this->service = $service;
		$this->environment = $environment;
	}
	
	public function validate(&$data)
    {
        //parent::verifyIsEmpty($data);
        
        foreach ($data as &$row) {
            parent::verifyIsEmpty($row, $row["ROWNUM"], ['CDFORMPGTO', 'NMFORMPGTO', 'IDTIPODTPGTO', 'IDTPPERIODO', 'IDREFECALCPARC', 'IDCONSDIAUTIL']);
            self::verifyCdFormPgto($row, $row["ROWNUM"]);
            self::verifyIdTipoDtPgto($row, $row["ROWNUM"]);
            self::verifyIdTipoPeriodo($row, $row["ROWNUM"]);
            self::verifyIdRefeCalcParc($row, $row["ROWNUM"]);
            self::verifyIdConsDiaUtil($row, $row["ROWNUM"]);
            
            self::verifyFormPgtoExists($row, $row["ROWNUM"]);

            $row['ERROS'] = $this->errors;
            
            parent::clearErrors();
        }
    }
    
    private function verifyIdConsDiaUtil(&$row, $rowNum) 
    {
        if (strlen($row['IDCONSDIAUTIL']) > 1) {
            parent::setError($rowNum, 'IDCONSDIAUTIL', $row['IDCONSDIAUTIL'], "1_CHAR_EXECEDED");
        }
        
        if (!strpos('_S_N_',$row['IDCONSDIAUTIL'])) {
            parent::setError($rowNum, 'IDCONSDIAUTIL', $row['IDCONSDIAUTIL'], "ONLY_ACCEPTS_S_N");
        }
    }
    
    private function verifyIdRefeCalcParc(&$row, $rowNum) 
    {
        if (!strpos('_DIA_DE_REFERENCIA_FORA_QUINZENA_FORA_SEMANA_FORA_DEZENA_FORA_MES_', $row['IDREFECALCPARC'])) {
            parent::setError($rowNum, 'IDREFECALCPARC', $row['IDREFECALCPARC'], "ONLY_ACCEPTS_IDREFECALCPARC");
        }
    }
    
    private function verifyIdTipoPeriodo(&$row, $rowNum) 
    {
        if (strlen($row['IDTPPERIODO']) > 1) {
            parent::setError($rowNum, 'IDTPPERIODO', $row['IDTPPERIODO'], "1_CHAR_EXECEDED");
        }
        
        if (!strpos('_M_S_A_',$row['IDTPPERIODO'])) {
            parent::setError($rowNum, 'IDTPPERIODO', $row['IDTPPERIODO'], "ONLY_ACCEPTS_M_S_A");
        }
    }
    
    private function verifyIdTipoDtPgto(&$row, $rowNum) 
    {
        if (strlen($row['IDTIPODTPGTO']) > 4) {
            parent::setError($rowNum, 'IDTIPODTPGTO', $row['IDTIPODTPGTO'], "4_CHAR_EXECEDED");
        }
        
        if (!strpos('_EMIN_EMEX_ENIN_ENEX_DSIN_DSEX_',$row['IDTIPODTPGTO'])) {
            parent::setError($rowNum, 'IDTIPODTPGTO', $row['IDTIPODTPGTO'], "ONLY_ACCEPTS_IDTIPODTPGTO");
        }
    }
    
    private function verifyCdFormPgto(&$row, $rowNum) 
    {
        $row['CDFORMPGTO'] = self::formatColumn($row['CDFORMPGTO'], 2);
        
        if (strlen($row['CDFORMPGTO']) > 2) {
            parent::setError($rowNum, 'CDFORMPGTO', $row['CDFORMPGTO'], "2_CHAR_EXECEDED");
        }
    }
    
    private function verifyFormPgtoExists($row, $rowNum)
    {
        if ($this->service->verifyFormPgtoExists($row)) {
            parent::setError($rowNum, 'CDFORMPGTO', $row['CDFORMPGTO'], "ALREADY_REGISTERED");
        }
    }
    
    public function formatToView(&$data)
    {
        foreach ($data as &$row) {
           $row['CDFORMPGTO'] = parent::formatColumn($row['CDFORMPGTO'], 2);
           $row['IDERROR'] = !empty($row['ERROS']) ? 'S' : 'N';
        }
    }
    
    public function save($data)
    {
        self::validate($data);
        parent::verifyExistErros($data);
        $this->service->saveImport($data);
    }
}